#include <iostream>
#include <vector>
using namespace std;

const size_t SCREEN_WIDTH = 80;
const size_t MAX_ASTERISK = SCREEN_WIDTH - 3 - 1;

struct Input {
    vector<double> numbers;
    size_t bin_count;
};

Input input_data() {
    size_t number_count;
    cerr << "Enter number count: ";
    cin >> number_count;

    Input in;
    in.numbers.resize(number_count);

    cerr << "Enter numbers: ";
    for (size_t i = 0; i < number_count; i++) {
        cin >> in.numbers[i];
    }

    cerr << "Enter bin count: ";
    cin >> in.bin_count;

    return in;
}

void find_minmax(const vector<double>& numbers, double& min, double& max) {
    min = numbers[0];
    max = numbers[0];
    for (double x : numbers) {
        if (x < min) {
            min = x;
        }
        if (x > max) {
            max = x;
        }
    }
}

vector<size_t> make_histogram(const vector<double>& numbers, size_t bin_count) {
    double min, max;
    find_minmax(numbers, min, max);

    vector<size_t> bins(bin_count, 0);
    double bin_size = (max - min) / bin_count;

    for (double number : numbers) {
        size_t bin_index = bin_count - 1;  // default to last bin
        if (number < max) {
            bin_index = static_cast<size_t>((number - min) / bin_size);
        }
        bins[bin_index]++;
    }

    return bins;
}

void show_histogram_text(const vector<size_t>& bins) {
    size_t max_count = bins[0];
    for (size_t x : bins) {
        if (x > max_count) {
            max_count = x;
        }
    }

    if (max_count > MAX_ASTERISK) {
        for (size_t count : bins) {
            size_t height = MAX_ASTERISK * (static_cast<double>(count) / max_count);
            if (count < 10) {
                cout << "  " << count << "|";
            } else if (count < 100) {
                cout << " " << count << "|";
            } else {
                cout << count << "|";
            }
            for (size_t i = 0; i < height; i++) {
                cout << "*";
            }
            cout << "\n";
        }
    } else {
        for (size_t x : bins) {
            if (x < 10) {
                cout << "  " << x << "|";
            } else if (x < 100) {
                cout << " " << x << "|";
            } else {
                cout << x << "|";
            }
            for (size_t i = 0; i < x; i++) {
                cout << "*";
            }
            cout << "\n";
        }
    }
}

int main() {
    auto in = input_data();
    auto bins = make_histogram(in.numbers, in.bin_count);
    show_histogram_text(bins);
    return 0;
}
